#include <iostream>

using namespace std;

class Polynomial {
  private:
    unsigned int degree;
    double * coefficients;
  public:
	  // Operator [] dla wielomianu
    double & operator[](unsigned int i)
    {
      return coefficients[i];
    }
	// Przeladowanie operatora ()
    double operator()(double x) 
    {
      double v=0.0;
      for(unsigned int i=degree; i>0; i--) v=x*v+coefficients[i];
      v=x*v+coefficients[0];
      return v;
    }

	// Dodawanie dwoch wielomianow
    const Polynomial operator+(const Polynomial & p) const 
    {
      unsigned int max, min;
      bool ch = false;
      if( degree>p.degree ) { max=degree; min=p.degree; ch=true;}
      else { max=p.degree; min=degree; }
      Polynomial c(max);
      for(unsigned int i=0; i<=min; i++ ) c.coefficients[i]=coefficients[i]+p.coefficients[i];
      for(unsigned int i=min+1; i<=max; i++ ) 
        if( ch ) c.coefficients[i]=coefficients[i];
        else c.coefficients[i]=p.coefficients[i];
      return c;
    }

	// Przeladowanie operatora = (przypisania)
    Polynomial & operator=(const Polynomial & p)
    {
      delete [] coefficients;
      degree=p.degree;
      coefficients=new double[degree+1];
      for(unsigned int i=0; i<=degree; i++) coefficients[i]=p.coefficients[i];
      return (*this);
    }

	// Mnozenie wielomianu przez liczbe (liczba * wielomian)
    friend const Polynomial operator*(double x, const Polynomial & p)
    {
      Polynomial c(p.degree);
      for(unsigned int i=0; i<=p.degree; i++) c.coefficients[i]=x*p.coefficients[i];
      return c;
    }

	// Mnozenie wielomianu przez liczbe (wielomian*liczba)
    const Polynomial operator*(double x) const
    {
      Polynomial c(degree);
      for(unsigned int i=0; i<=degree; i++) c.coefficients[i]=x*coefficients[i];
      return c;
    }

	// Tworzenie wielomianu. Ilosc wspolczynnikow zalezy od stopnia wielomianu
    Polynomial(unsigned int n) 
    {
      degree=n;
      coefficients=new double[n+1]; // x^3 + x^2 + x +1
      for(unsigned int i=0; i<n; i++) 
      {
        coefficients[i]=0.0;
      }
	  // Najwiekszy wspolczynnik jest 1.0, reszta 0.0
      coefficients[n]=1.0;
    }
    ~Polynomial()
    {
       delete [] coefficients;
    }

	// Wypisywanie wielomianu
    friend ostream & operator<<(ostream & out, const Polynomial & p)
    {
       bool first=false;
       for(unsigned int i=p.degree; i>1; i--) 
       {   
         if(first && p.coefficients[i]>0) out << "+";
         if(p.coefficients[i]!=0) { out << p.coefficients[i] << "*x^" << i; first=true; }
       }
       if(first && p.coefficients[1]>0) out << "+";
       if(p.coefficients[1]!=0) { out << p.coefficients[1] << "*x"; first=true; }
       if(first && p.coefficients[0]>0) out << "+";
       if(p.coefficients[0]!=0) out << p.coefficients[0];
       return out;
    }
};

int main(int argc, char* argv[])
{
  unsigned int n=2;

  Polynomial a(n), b(n+1), c(1);
  cout << "Wartosc a: "<<a << endl;
  cout << "Wartosc b: "<< b << endl;
  cout << "Wartosc c: "<< c << endl;
  
  b[0]=2.0;

  cout << "Wartosc b po modyfikacji: "<< b << endl;
  cout << "Wartosc b[0] po modyfikacji: "<< b[0] << endl;

  c = 3 * b;
  cout << "Wartosc c po modyfikacji: " <<  c << endl;
  // Uruchamia sie przeladowanie operatora ()
  cout << "Kalkulacja c(1.0): " << c(1.0) << endl;

  c = b * 2;
  cout << "Wartosc c po kolejnej modyfikacji: " <<  c << endl;
  cout << "Kalkulacja c(1.0) po modyfikacji: " << c(1.0) << endl;

  Polynomial d = a + b + c;
 

  cout << "Wartosc d po dodawaniu: " << d << endl;

  cout << "Kalkulacja d(1.0): " << d(1.0) << endl;
  cout << "Kalkulacja d(2.0): " << d(2.0) << endl;

  cout << "Wartosc a: " << a << endl;
  cout << "Kalkulacja a(1.0): " << a(1.0) << endl;
  a = b;
  cout << "Wartosc a po przypisaniu b: " << a << endl;
  cout << "Kalkulacja a(1.0): " << a(1.0) << endl; 
  
  return 0;
}

